<?php

// $Id: client.php 26444 2008-12-11 16:19:29Z wsl $
// $URL: https://infix.uvt.nl/its-id/trunk/sources/aselect-php/src/client.php $

function aselect_split($in) {
	$pairs = explode('&', $in);
	$ret = array();
	foreach($pairs as $pair) {
		$keyval = explode('=', $pair, 2);
		if(count($keyval) < 2)
			return NULL;
		$ret[urldecode($keyval[0])] = urldecode($keyval[1]);
	}
	return $ret;
}

function aselect_join($pairs) {
	$ret = array();
	foreach($pairs as $key => $val) {
		$ret[] = rawurlencode($key).'='.rawurlencode($val);
	}
	return implode('&', $ret);
}

function aselect_valid($a) {
	foreach($a as $key => $val) {
		if(is_null($key) || is_null($val))
			return false;
		if(ereg('[^ -~]', $key) || ereg('[^ -~]', $key))
			return false;
	}
	return true;
}

function aselect_comm_io($agent, $msg) {
	if(!socket_connect($agent, '127.0.0.1', 1495))
		return null;
	$r = socket_write($agent, $msg);
//	error_log("aselect-client: send: $msg");
	if($r === false || $r < strlen($msg))
		return null;
	socket_shutdown($agent, 1);
	$ret = socket_read($agent, 65536);
	if($ret === false || $ret == '')
		return null;
//	error_log("aselect-client: recv: $ret");
	$ret = aselect_split($ret);
	if(is_null($ret))
		return null;
	$code = $ret['result_code'];
	if(strcmp($code, '0000'))
		return null;
	unset($ret['result_code']);
	return $ret;
}

function aselect_comm($a) {
	$msg = aselect_join($a);
	$agent = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
	if(!$agent)
		return null;
	$ret = aselect_comm_io($agent, $msg);
	socket_close($agent);
	return $ret;
}

function aselect_url($url, $a) {
	$sep = strpos($url, '?') === false ? '?' : '&';
	$ret = array($url);
	foreach($a as $key => $val) {
		$ret[] = $sep;
		$ret[] = urlencode($key);
		$ret[] = '=';
		$ret[] = urlencode($val);
		$sep = '&';
	}
	return implode('', $ret);
}

function aselect_defaulturl() {
	$scheme = $_SERVER['HTTPS'] == 'on' ? 'https' : 'http';
	return $scheme.'://'.$_SERVER["HTTP_HOST"].$_SERVER['REQUEST_URI'];
}

function aselect_authenticate($app_id, $app_url = null) {
	if(is_null($app_url))
		$app_url = aselect_defaulturl();
	$args = array(
			'request' => 'authenticate',
			'app_id' => $app_id,
			'app_url' => $app_url
		);
	$res = aselect_comm($args);
	if(is_null($res))
		return null;
	$as_url = $res['as_url'];
	if(is_null($as_url))
		return null;
	unset($res['as_url']);
	return aselect_url($as_url, $res);
}

function aselect_verify_credentials($rid, $cred) {
	$args = array(
			'request' => 'verify_credentials',
			'rid' => $rid,
			'aselect_credentials' => $cred
		);
	if(!aselect_valid($args))
		return null;
	$res = aselect_comm($args);
	if(is_null($res))
		return null;
	return aselect_join(array(
			'uid' => $res['uid'],
			'organization' => $res['organization'],
			'ticket' => $res['ticket']
		));
}

function aselect_verify_ticket($cookie) {
	$res = aselect_split($cookie);
	$args = array(
			'request' => 'verify_ticket',
			'uid' => $res['uid'],
			'organization' => $res['organization'],
			'ticket' => $res['ticket']
		);
	if(!aselect_valid($args))
		return null;
	$res = aselect_comm($args);
	if(is_null($res))
		return null;
	return array(
			'uid' => $args['uid'],
			'organization' => $args['organization'],
			'ticket' => $args['ticket']
		);
}

?>
